"""View containers for wrkmon TUI."""

from wrkmon.ui.views.search import SearchView
from wrkmon.ui.views.queue import QueueView
from wrkmon.ui.views.history import HistoryView
from wrkmon.ui.views.playlists import PlaylistsView

__all__ = ["SearchView", "QueueView", "HistoryView", "PlaylistsView"]
