"""TUI screens for wrkmon."""

from wrkmon.ui.screens.search import SearchScreen
from wrkmon.ui.screens.player import PlayerScreen
from wrkmon.ui.screens.playlist import PlaylistScreen
from wrkmon.ui.screens.history import HistoryScreen

__all__ = ["SearchScreen", "PlayerScreen", "PlaylistScreen", "HistoryScreen"]
